# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinContentSupport library

    @copyright: 2008, 2009, 2010, 2011 by Paul Boddie <paul@boddie.org.uk>
    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2005-2008 MoinMoin:ThomasWaldmann.
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.wikiutil import escape
import re

__version__ = "0.1"

# Regular expressions.
# NOTE: These overlap with ImprovedMoinSearch and EventAggregator.

heading_regexp = re.compile(r"^(?P<level>=+)\s*(?P<heading>.*?)\s*(?P=level)$", re.UNICODE | re.MULTILINE)

category_membership_str = ur"^\s*(?:(Category\S+)(?:\s+(Category\S+))*)\s*$"
category_membership_regexp = re.compile(category_membership_str, re.MULTILINE | re.UNICODE)
category_declarations_regexp = re.compile("^----$\s*" + category_membership_str, re.MULTILINE | re.UNICODE)

def getHeadingDetails(body, min_level=None, max_level=None):

    """
    Return heading details from the given 'body' for headings with the given
    'min_level' to 'max_level' range. Specifying None or omitting 'max_level' or
    'min_level' removes the appropriate constraint on the range.

    A list of tuples containing the heading, the level, and the span (the start
    offset and the end offset as a tuple) is returned.
    """

    headings = []

    for match in heading_regexp.finditer(body):
        level = len(match.group("level"))

        if (min_level is None or min_level <= level) and \
            (max_level is None or level <= max_level):

            headings.append((match.group("heading"), level, match.span()))

    return headings

def getCategoryMembership(body):

    "From the given 'body', return the categories the page belongs to."

    match = category_membership_regexp.search(body)
    if match:
        return [x for x in match.groups() if x]
    else:
        return []

def getCategoryDeclarations(body):

    """
    From the given 'body', return the category declaration sections in the page
    in the form of a list of tuples, each containing a list of categories, the
    start of the declaration region and the end of the region.
    """

    return [([x for x in match.groups() if x], match.span()) for match in category_declarations_regexp.finditer(body)]

def makeCategoryDeclaration(categories):

    "Return a category declaration string for the given 'categories'."

    return "\n----\n%s\n" % " ".join(categories)

# Utility classes and associated functions.
# NOTE: These are a subset of EventAggregatorSupport.

class Form:

    """
    A wrapper preserving MoinMoin 1.8.x (and earlier) behaviour in a 1.9.x
    environment.
    """

    def __init__(self, form):
        self.form = form

    def get(self, name, default=None):
        values = self.form.getlist(name)
        if not values:
            return default
        else:
            return values

    def __getitem__(self, name):
        return self.form.getlist(name)

class ActionSupport:

    """
    Work around disruptive MoinMoin changes in 1.9, and also provide useful
    convenience methods.
    """

    def get_form(self):
        return get_form(self.request)

def get_form(request):

    "Work around disruptive MoinMoin changes in 1.9."

    if hasattr(request, "values"):
        return Form(request.values)
    else:
        return request.form

class send_headers:

    """
    A wrapper to preserve MoinMoin 1.8.x (and earlier) request behaviour in a
    1.9.x environment.
    """

    def __init__(self, request):
        self.request = request

    def __call__(self, headers):
        for header in headers:
            parts = header.split(":")
            self.request.headers.add(parts[0], ":".join(parts[1:]))

def escattr(s):
    return escape(s, 1)

# vim: tabstop=4 expandtab shiftwidth=4
